import { world } from "mojang-minecraft";
import { ModalFormData, ActionFormData } from "mojang-minecraft-ui";

var modals = {};

world.events.beforeItemUseOn.subscribe( async data => {
	const { blockLocation, item, source } = data;
    const { x, y, z } = blockLocation;
    const { dimension } = source;
    const block = dimension.getBlock(blockLocation);

    let modalid = `${dimension.id.split(':')[1]}:${x}:${y}:${z}`;

    if(modals[modalid])
        return;
    
    modals[modalid] = true;

    if(block.id == 'genshin:genshinwaypoint'){
        try {
            const modal = new ModalFormData()
            .title('Enter waypoint name')
            .textField('','');

            const result = await modal.show(source);

            if (!result.isCanceled) {
                result.formValues.forEach(value=>{
                    var str = value;

                    str = str.replaceAll( ' ', '_');
                    str = str.replaceAll( ':', '');

                    source.addTag(`genshinway:${str}:${dimension.id.split(':')[1]}:${x}:${y}:${z}`);
                });

                source.runCommand(`setblock ${x} ${y} ${z} genshin:genshinwaypointon`);
            };

            delete modals[modalid];
        } catch (error) {
            source.runCommand("say Your error was: " + error);
            delete modals[modalid];
        }
    } else
    if(block.id == 'genshin:genshinwaypointon'){
        try {
            const form = new ActionFormData()
                    .title("Waypoints");
            
            let ways = [];

            source.getTags().forEach(tag=>{
                if(tag.includes('genshinway')){
                    const waypoint = tag.replaceAll('_',' ').split(':');

                    ways.push({
                        'dimension': waypoint[waypoint.length-4],
                        'x': waypoint[waypoint.length-3],
                        'y': waypoint[waypoint.length-2],
                        'z': waypoint[waypoint.length-1]
                    });

                    form.button(waypoint[waypoint.length-5]);
                }
            });

            if(ways.length < 1)
                return;

            const result = await form.show(source);

            if (!result.isCanceled) {
                const way = ways[result.selection];

                world.getDimension(way.dimension).runCommand(`tp "${source.nameTag}" ${way.x} ${way.y} ${way.z}`);
            };

            delete modals[modalid];
        } catch (error) {
            source.runCommand("say Error: " + error);
            delete modals[modalid];
        }
    }
});

world.events.blockBreak.subscribe( data => {
	const { block, dimension, player, brokenBlockPermutation } = data;
	const { location } = block;
	const { x, y, z } = location;
	const r = 1;

	if(brokenBlockPermutation.type.id == 'genshin:genshinwaypointon'){
        const sub_id = `:${dimension.id.split(':')[1]}:${x}:${y}:${z}`;

		Array.from(world.getPlayers()).forEach(player => {
            player.getTags().forEach(tag=>{
                if(tag.includes(sub_id)){
                    player.removeTag(tag);
                }
            })
        })
	}
});

world.events.beforeChat.subscribe( data => {
    const { sender } = data;

    if(data.message.includes('removemywaypoint')){
        const wayid = data.message.split(':')[1];

        if(sender.hasTag(wayid)){
            sender.removeTag(wayid);
            sender.runCommand(`tell "${sender.nameTag}" Waypoint removed: ${wayid}`);
        } else {
            sender.runCommand(`tell "${sender.nameTag}" Waypoint does not exist: ${wayid}`);
        }
    } else {
        switch(data.message){
            case 'clearmywaypoints':
                sender.getTags().forEach(tag=>{
                    if(tag.includes('genshinway'))
                        sender.removeTag(tag)
                });
                
                sender.runCommand(`tell "${sender.nameTag}" Your waypoints have been removed.`);
                break;
            case 'listmywaypoints':
                sender.getTags().forEach(tag=>{
                    if(tag.includes('genshinway'))
                        sender.runCommand(`tell "${sender.nameTag}" ${tag}`);
                });
                break;
        }
    }
});